@@ -29,8 +29,9 @@ Rcpp::S4 CPP_coverage_fraction(Rcpp::S4 & rast, const Rcpp::RawVector & wkb, boo
   try {
     GEOSAutoHandle geos;
     Rcpp::Environment raster = Rcpp::Environment::namespace_env("raster");
+    Rcpp::Environment xx = Rcpp::Environment::namespace_env("exactextractr");
     Rcpp::Function rasterFn = raster["raster"];
-    Rcpp::Function crsFn = raster["crs"];
+    Rcpp::Function crsFn = xx[".crs"];
 
     auto grid = make_grid(rast);
     auto coverage_fraction = raster_cell_intersection(grid, geos.handle, read_wkb(geos.handle, wkb).get());
